package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.risk.control.common.enums.ExceptionEnum;
import com.tzld.piaoquan.risk.control.common.enums.RedisPrefixEnum;
import com.tzld.piaoquan.risk.control.common.exception.CommonException;
import com.tzld.piaoquan.risk.control.component.RedisUtils;
import com.tzld.piaoquan.risk.control.dao.mapper.UserMapper;
import com.tzld.piaoquan.risk.control.model.dto.LoginDTO;
import com.tzld.piaoquan.risk.control.model.param.LoginAndRegisterParam;
import com.tzld.piaoquan.risk.control.model.po.User;
import com.tzld.piaoquan.risk.control.model.po.UserExample;
import com.tzld.piaoquan.risk.control.service.UserService;
import com.tzld.piaoquan.risk.control.util.TokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Resource
    private RedisUtils redisUtils;

    @Override
    public LoginDTO login(LoginAndRegisterParam param) {

        if (StringUtils.isAnyBlank(param.getAccount(), param.getPassword())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }

        UserExample example = new UserExample();
        if (this.isValidPhone(param.getAccount())) {
            example.createCriteria().andPhoneEqualTo(param.getAccount());
        } else if (this.isValidEmail(param.getAccount())) {
            example.createCriteria().andEmailEqualTo(param.getAccount());
        } else {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        List<User> users = userMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(users)) {
            throw new CommonException(ExceptionEnum.ACCOUNT_NOT_EXIST);
        }

        User user = users.get(0);

        if (!passwordEncoder.matches(param.getPassword(), user.getPassword())) {
            throw new CommonException(ExceptionEnum.ILLEGAL_OPERATION, "密码错误");
        }

        // 设置Token缓存
        String token = TokenUtil.genTokenByUUIDFormat();
        String key = RedisPrefixEnum.RISK_CONTROL_TOKEN.getPrefix().replace("{token}", token);
        user.setPassword(null);
        redisUtils.set(key, JSON.toJSONString(user), 24 * 60 * 60);

        LoginDTO loginDTO = new LoginDTO();
        loginDTO.setToken(token);
        loginDTO.setName(user.getName());
        return loginDTO;
    }

    @Override
    public void register(LoginAndRegisterParam param) {

        if (StringUtils.isAnyBlank(param.getAccount(), param.getPassword())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }

        User user = new User();

        UserExample example = new UserExample();
        if (this.isValidPhone(param.getAccount())) {
            example.createCriteria().andPhoneEqualTo(param.getAccount());
            user.setPhone(param.getAccount());
        } else if (this.isValidEmail(param.getAccount())) {
            example.createCriteria().andEmailEqualTo(param.getAccount());
            user.setEmail(param.getAccount());
        } else {
            throw new CommonException(ExceptionEnum.DATA_ERROR);
        }

        long count = userMapper.countByExample(example);
        if (count > 0) {
            throw new CommonException(ExceptionEnum.ACCOUNT_EXIST);
        }
        String newPassword = passwordEncoder.encode(param.getPassword());
        user.setPassword(newPassword);
        user.setName(param.getName());
        userMapper.insertSelective(user);
    }

    // 邮箱正则校验
    private boolean isValidEmail(String email) {
        // 实现邮箱正则校验的逻辑
        String emailRegex = "^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,}$";
        return email.matches(emailRegex);
    }

    // 手机号正则校验
    private boolean isValidPhone(String phoneNumber) {
        String phoneRegex = "^1[3456789]\\d{9}$";
        return phoneNumber.matches(phoneRegex);
    }


}
