package com.tzld.piaoquan.risk.control.service.rule;

import lombok.Getter;

import java.util.Map;

/**
 * 比较
 *
 * @author supeng
 */
@Getter
public class Comparison implements Expression {
    private final String leftVar;
    private final Double leftNum;
    private final TokenType operator;
    private final String rightVar;
    private final Double rightNum;

    public Comparison(String leftVar, Double leftNum, TokenType operator, String rightVar, Double rightNum) {
        this.leftVar = leftVar;
        this.leftNum = leftNum;
        this.operator = operator;
        this.rightVar = rightVar;
        this.rightNum = rightNum;
    }

    @Override
    public boolean evaluate(Map<String, Double> context) {
        double left = leftVar != null ? context.getOrDefault(leftVar, 0.0) : leftNum;
        double right = rightVar != null ? context.getOrDefault(rightVar, 0.0) : rightNum;
        switch (operator) {
            case GT:
                return left > right;
            case LT:
                return left < right;
            case EQ:
                return left == right;
            case GTE:
                return left >= right;
            case LTE:
                return left <= right;
            case NEQ:
                return left != right;
            default:
                throw new UnsupportedOperationException("Unsupported operator: " + operator);
        }
    }
}
